/*
 * acompaction.cpp
 *
 *  Created on: 2. 11. 2014
 *      Author: Tomas Pecka
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <sax/FromXMLParserHelper.h>

#include "exception/AlibException.h"
#include "factory/XmlDataFactory.hpp"
#include "automaton/transform/Compaction.h"

int main(int argc, char** argv) {

	try {
		TCLAP::CmdLine cmd("Automaton compaction binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Automaton to compactify",		false,	"-",		"file");
		cmd.add( input );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		measurements::start("Overal", measurements::Type::OVERALL);
		measurements::start("Input read", measurements::Type::AUXILIARY);

		automaton::Automaton automaton = alib::XmlDataFactory::fromTokens<automaton::Automaton>(sax::FromXMLParserHelper::parseInput(input));

		measurements::end();
		measurements::start("Algorithm", measurements::Type::MAIN);

		automaton::Automaton res = automaton::transform::Compaction::convert(automaton);

		measurements::end();
		measurements::start("Output write", measurements::Type::AUXILIARY);

		alib::XmlDataFactory::toStdout(res);

		measurements::end();
		measurements::end();

		if(measure.getValue()) std::cmeasure << measurements::results() << std::endl;

		return 0;
	} catch (const exception::AlibException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cout << exception.error() << std::endl;
		return 2;
	} catch(...) {
		return 127;
	}
}
